package massim.competition2015;

import massim.competition2015.configuration.MapSimulationConfiguration;
import massim.competition2015.scenario.TeamState;
import massim.framework.Observer;
import massim.framework.SimulationConfiguration;
import massim.framework.SimulationState;
import massim.framework.simulation.SimulationStateImpl;
import massim.framework.util.DebugLog;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;
import java.util.stream.Collectors;

public class GraphSimulationStatisticsObserver implements Observer {
	
	private MapSimulationWorldState worldState;
	private String outputPath;

	public GraphSimulationStatisticsObserver() {
		super();
	}

	public GraphSimulationStatisticsObserver(String outputpath) {
        super();
		this.outputPath = outputpath;
	}

	@Override
	public void start() {
	}

	@Override
	public void stop() {
	}

	@Override
	public void notifySimulationStart(){}

	@Override
	public void notifySimulationEnd() {

	    File dir = new File(outputPath);
        dir.mkdirs();

		// final output
		DebugLog.log(DebugLog.LOGLEVEL_NORMAL, "Generating Results File...");

        String fileName = new SimpleDateFormat("y-MM-dd-hh-mm-ss").format(new Date())
                + "_" + worldState.simulationName + ".xml";
        Vector<TeamState> teams = new Vector<>(
                worldState.agents.stream().map(agent -> agent.teamState).collect(Collectors.toSet()));

        try {
            Document doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
            Element root = doc.createElement("result");
            doc.appendChild(root);

            for(int i = 0; i <= 1; i++){
                TeamState team = teams.get(i);
                TeamState opponent = teams.get(1-i);
                Element elTeam = doc.createElement("team");
                root.appendChild(elTeam);
                elTeam.setAttribute("name", team.name);
                elTeam.setAttribute("money", String.valueOf(team.money));
                int points = 0;
                if(team.money == opponent.money) points = 1;
                else if(team.money > opponent.money){
                    points = 3;
                    if(team.money <= worldState.config.seedcapital) points--;
                }
                elTeam.setAttribute("points", String.valueOf(points));
            }

            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
            DOMSource source = new DOMSource(doc);
            StreamResult result = new StreamResult(new FileWriter(outputPath + File.separator + fileName));
            transformer.transform(source, result);

        } catch (ParserConfigurationException | TransformerException | IOException e) {
            e.printStackTrace();
        }
    }

	@Override
	public synchronized void notifySimulationState(SimulationState state) {

		SimulationStateImpl tcstate = (SimulationStateImpl) state;
		MapSimulationWorldState wstate = (MapSimulationWorldState) tcstate.simulationState;
		if (wstate.currentStep > -1) {
			if (this.worldState == null) {
				this.worldState = wstate;
			}
		}
	}

	@Override
	public void notifySimulationConfiguration(SimulationConfiguration simconf) {
		MapSimulationConfiguration conf = (MapSimulationConfiguration) simconf;
	}

}
